
class SpeckleMesh:
	@staticmethod
	def Create(resource):
		if hasattr(resource, 'type'):
			if resource.type == "Mesh":
				return SpeckleMesh(resource)
		return None

	self.vertices = []
	self.faces = []
	self.uvs = []
	self.colors = []

	def __init__(self, resource):
		if not hasattr(resource, 'vertices'):
			raise AssertionError("Resource does not contain vertex list.")
		if not hasattr(resource, 'faces'):
			raise AssertionError("Resource does not contain face list.")

	    if hasattr(o, 'properties') and o.properties is not None and hasattr(o.properties, 'texture_coordinates'):
	        decoded = base64.b64decode(o.properties.texture_coordinates).decode("utf-8")
	        s_uvs = decoded.split()
	          
	        if len(s_uvs) * 2 == len(o.vertices):
	            for i in range(0, len(s_uvs), 2):
	                self.uvs.append((float(s_uvs[i]), float(s_uvs[i+1])))
	        else:
	            print ("Failed to match UV coordinates to vert data.")
	    
	    if hasattr(o, 'vertices') and len(o.vertices) > 0:
	        for i in range(0, len(o.vertices), 3):
	            self.vertices.append((float(o.vertices[i]), float(o.vertices[i + 1]), float(o.vertices[i + 2])))

	    if hasattr(o, 'colors') and len(o.colors) > 0:
	        for i in range(0, len(o.colors), 3):
	            self.colors.append((int(o.colors[i]), int(o.colors[i + 1]), int(o.colors[i + 2])))

	    if hasattr(o, 'faces') and len(o.faces) > 0:
	        i = 0
	        while (i < len(o.faces)):
	            if (o.faces[i] == 0):
	                i += 1
	                self.faces.append((int(o.faces[i]), int(o.faces[i + 1]), int(o.faces[i + 2])))
	                i += 3
	            elif (o.faces[i] == 1):
	                i += 1
	                self.faces.append((int(o.faces[i]), int(o.faces[i + 1]), int(o.faces[i + 2]), int(o.faces[i + 3])))
	                i += 4
	            else:
	                print("Invalid face length.\n" + str(o.faces[i]))
	                break



import requests, json, os
import struct, base64

from .SpeckleResource import SpeckleResource

'''
- SpeckleClient payloads are always SpeckleResource objects, never dictionaries
- SpeckleClient returns are always SpeckleReousrce objects, never dictionaries
'''

class SpeckleClient(object):

    verbose = False

    def __init__(self):
        self.baseUrl = "https://hestia.speckle.works/api/v1"
        self.authToken = ""
        self.useGzip = True
        self.header = None

    '''
    Utility functions
    '''
    def log(self, msg):
        print ('SpeckleClient: %s' % msg)

    def check_response_status_code(self, r):
        # debug
        if self.verbose:
            print()
            self.log(json.dumps(r.json(), indent=4, sort_keys=True))

        # parse response
        if r.status_code == 200:
            self.log("Request successful: %s" % r.reason)
            return True
        elif r.status_code == 400:
            self.log("Request failed: %s" % r.reason)
            #print(r.text)
        elif r.status_code != 200 and r.status_code != 204:
            self.log("The HTTP status code of the response was not expected: %s, %s" % (r.status_code, r.reason))
        return False

    def CreateHeader(self):
        if self.header is None:
            self.header = {'content-type' : 'application/json', 'Authorization': self.authToken}
        return self.header

    def PrepareResponse(self, header):
        header = {'content-type' : 'application/json'}
        if self.authToken:
            header['Authorization'] = self.authToken

    def LoadProfiles(self):
        profiles = {}
        path = os.path.join(os.getenv('LOCALAPPDATA'), 'SpeckleSettings')
        if os.path.isdir(path):
            files = [os.path.join(path, x) for x in os.listdir(path) if x.endswith('.txt')]
            for file in files:
                with open(file, 'r') as f:
                    line = f.readline()
                    tokens = line.split(',')
                    profiles[tokens[0]] = {'name': tokens[2], 'authtoken': tokens[1], 'server': tokens[3]}

        return profiles

    def SaveNewProfile(self, profile):
        assert ("email" in profile.keys())
        assert ("server_name" in profile.keys())
        assert ("authtoken" in profile.keys())

        path = os.path.join(os.getenv('LOCALAPPDATA'), 'SpeckleSettings')
        if os.path.isdir(path):
            with open(os.path.join(path, profile['email']) + ".JWT .txt", 'w') as f:
                f.writeline("%s,%s,%s,%s,%s" % (profile['email'], profile['authtoken'], profile['server_name'], profile['server'], profile['server']))

    def UseExistingProfile(self, email):
        profiles = self.LoadProfiles()
        if email in profiles.keys():
            self.baseUrl = profiles[email]['server']
            self.authToken = profiles[email]['authtoken']
            return True
        return False

    '''
    Client methods
    '''
    def UserRegister(self, profile):
        assert ("server_name" in profile.keys())
        assert ("email" in profile.keys())
        assert ("password" in profile.keys())
        assert ("name" in profile.keys())
        assert ("surname" in profile.keys())

        url = self.baseUrl + "/accounts/register"

        headers = {'content-type' : 'application/json'}
        r = requests.post(url, data=json.dumps(profile), headers=headers)

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UserLogin(self, login_dict):
        assert ("email" in login_dict.keys())
        assert ("password" in login_dict.keys())

        url = self.baseUrl + "/accounts/login"

        headers = {'content-type' : 'application/json'}
        r = requests.post(url, data=json.dumps(login_dict), headers=headers)

        if self.check_response_status_code(r):
            login_response = r.json()
            #print(login_response['resource'].keys())
            self.authToken = login_response['resource']['apitoken']
            return login_response
        return None

    def GetStreams(self):
        url = self.baseUrl + "/streams"
        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UserClientsGet(self):
        url = self.baseUrl + "/accounts/clients"

        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UserGetProfile(self):
        url = self.baseUrl + "/accounts/profile"

        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None     

    def UserUpdateProfile(self, update_dict):
        assert ("email" in update_dict.keys())
        assert ("name" in update_dict.keys())
        assert ("surname" in update_dict.keys())
        assert ("company" in update_dict.keys())

        url = self.baseUrl + "/accounts/profile"
        r = requests.put(url, data=json.dumps(login_dict), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None         

    def ClientCreate(self, create_dict):
        assert ("client" in create_dict.keys())

        url = self.baseUrl + "/clients"
        r = requests.post(url, data=json.dumps(create_dict), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def ClientGet(self, clientId):
        if not clientId: raise

        url = self.baseUrl + "/clients/%s" % clientId
        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def ClientUpdate(self, update_dict, clientId):
        assert ("client" in update_dict.keys())

        if not clientId: raise

        url = self.baseUrl + "/clients/%s" % clientId
        r = requests.put(url, data=json.dumps(update_dict), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None         

    def ClientDelete(self, clientId):
        if not clientId: raise

        url = self.baseUrl + "/clients/%s" % clientId
        r = requests.delete(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def StreamCreate(self, name="Speckle Stream", layers=None):
        create_dict = {"name" : name}
        if layers is None:
            create_dict['layers'] = ""
        else:
            create_dict['layers'] = layers

        assert ("layers" in create_dict.keys())
        assert ("name" in create_dict.keys())

        url = self.baseUrl + "/streams"
        r = requests.post(url, data=json.dumps(create_dict), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def StreamDelete(self, streamId):
        url = self.baseUrl + "/streams/%s" % streamId
        r = requests.delete(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def GetStreamObjects(self, streamId, query = None):
        assert streamId is not None
        url = self.baseUrl + "/streams/%s/" % streamId
        if query is not None:
            url = url + "?" + query

        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def ObjectCreate(self, obj):
        url = self.baseUrl + "/objects"
        payload = SpeckleResource.to_dict(obj)

        if '_id' in payload.keys():
            del payload['_id']
        if 'hash' in payload.keys():
            del payload['hash']

        r = requests.post(url, json.dumps(payload), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def ObjectToStream(self, objectId, streamId):
        assert objectId is not None
        assert streamId is not None

        url = self.baseUrl + "/streams/%s" % streamId

        res = GetStreamObjects(self, None, streamId)

        r = requests.put(url, json.dumps({"objects":[objectId]}), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None 

    def AddObjects(self, obj, streamId):
        assert streamId is not None
        if len(obj) == -1:
            payload = {"object":obj[0]} if isinstance(obj[0], dict) else {"object":obj[0].__dict__}
        else:
            payload = {"objects":[x if isinstance(x, dict) or isinstance(x, str) else x.__dict__ for x in obj ]}
        #payload = {"objects":[obj]}
        #self.log (json.dumps(payload, indent=4, sort_keys=True))

        url = self.baseUrl + "/streams/%s" % streamId
        r = requests.put(url, json.dumps(payload), headers=self.CreateHeader())
        #print(r.text)

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UpdateLayers(self, layers, streamId):
        assert streamId is not None
        payload = {"layers":[x if isinstance(x, dict) or isinstance(x, str) else x.__dict__ for x in layers ]}
        #payload = {"objects":[obj]}
        #self.log (json.dumps(payload, indent=4, sort_keys=True))

        url = self.baseUrl + "/streams/%s" % streamId
        r = requests.put(url, json.dumps(payload), headers=self.CreateHeader())
        #print(r.text)

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UpdateStream(self, stream, streamId):
        assert streamId is not None
        url = self.baseUrl + "/streams/%s" % streamId
        r = requests.put(url, SpeckleResource.to_json(stream), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None        

    def UpdateObject(self, obj):
        assert obj._id is not None

        url = self.baseUrl + "/objects/%s" % obj._id

        '''
        if isinstance(obj, dict):
            payload = obj
        else:
            payload = obj.__dict__
        '''

        r = requests.put(url, SpeckleResource.to_json(obj), headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def DeleteObject(self, obj_id):
        assert obj_id is not None

        url = self.baseUrl + "/objects/%s" % obj_id
        r = requests.delete(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None        

    def GetObject(self, objectId):
        assert objectId is not None

        url = self.baseUrl + "/objects/%s" % objectId
        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def GetObjectBulk(self, objectIds, query):
        url = self.baseUrl + "/objects/getbulk?%s" % query
        r = requests.post(url, data=json.dumps(objectIds), headers=self.CreateHeader())

        print("Shit: " + r.url)

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None
  

    def GetLayers(self, streamId):
        assert streamId is not None

        url = self.baseUrl + "/streams/%s?fields=layers" % streamId
        print (url)
        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def GetSingleLayer(self, streamId, layerId):
        assert streamId is not None
        assert layerId is not None

        url = self.baseUrl + "/streams/%s/layers/%s" % (streamId, layerId)
        r = requests.get(url, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

    def UpdateSingleLayer(self, body, streamId, layerId):
        assert streamId is not None
        assert layerId is not None
        assert 'layer' in body.keys()

        url = self.baseUrl + "/streams/%s/layers/%s" % (streamId, layerId)
        r = requests.patch(url, body, headers=self.CreateHeader())

        if self.check_response_status_code(r):
            return SpeckleResource(r.json())
        return None

if __name__ == '__main__':
    s = SpeckleClient()
    profiles = s.LoadProfiles()
    print (json.dumps(profiles, indent=4, sort_keys=True))
    s.UseExistingProfile(profiles[0])

    res = s.GetStreams()

    if res is not None:
        for i in res['resources']:
            self.log(i['streamId'])
            objects = s.GetStreamObjects(None, i['streamId'])
            print (json.dumps(objects, indent=4, sort_keys=True))
            print()






import json

class SpeckleResource(object):
    def __init__(self, d):
        for a, b in d.items():
            if isinstance(b, (list, tuple)):
               setattr(self, a, [SpeckleResource(x) if isinstance(x, dict) else x for x in b])
            else:
               setattr(self, a, SpeckleResource(b) if isinstance(b, dict) else b)

    @staticmethod
    def to_dict(obj):
        '''
        from https://stackoverflow.com/questions/1036409/recursively-convert-python-object-graph-to-dictionary
        '''
        if isinstance(obj, dict):
            data = {}
            for (k, v) in obj.items():
                data[k] = SpeckleResource.to_dict(v)
            return data
        elif isinstance(obj, (list, tuple)):
            return [SpeckleResource.to_dict(v) for v in obj]
        #elif hasattr(obj, "_ast"):
        #    return SpeckleResource.to_dict(obj._ast())
        #elif hasattr(obj, "__iter__"):
        #    return [SpeckleResource.to_dict(v) for v in obj]        
        elif hasattr(obj, "__dict__"):
            data = dict([(key, SpeckleResource.to_dict(value))
                         for key, value in obj.__dict__.items()
                         #if not callable(value) and not key.startswith('_') and key not in ['_name']])
                         if not callable(value)])
            #if hasattr(obj, "__class__"):
            #    data['SpeckleResource'] = obj.__class__.__name__
            return data
        else:
            return obj

    @staticmethod
    def to_json(obj):
        return json.dumps(SpeckleResource.to_dict(obj), sort_keys=True)

    @staticmethod
    def to_json_pretty(obj):
        return json.dumps(SpeckleResource.to_dict(obj), indent=4, sort_keys=True)


    @staticmethod
    def isSpeckleObject(obj):
        if (hasattr(obj, 'type') and
        hasattr(obj, '_id') and
        hasattr(obj, 'hash') and
        hasattr(obj, 'geometryHash') and
        hasattr(obj, 'applicationId') and
        hasattr(obj, 'properties')):
            return True
        return False

    @staticmethod
    def createSpeckleObject():
        attr = {'type':None,'_id':None, 'hash':1234, 'geometryHash':None,'applicationId':None, 'properties':None}
        return SpeckleResource(attr)

    @staticmethod
    def isSpeckleMesh(obj):
        if (hasattr(obj, 'type') and
        hasattr(obj, '_id') and
        hasattr(obj, 'hash') and
        hasattr(obj, 'geometryHash') and
        hasattr(obj, 'applicationId') and
        hasattr(obj, 'vertices') and
        hasattr(obj, 'faces') and
        hasattr(obj, 'colors') and
        hasattr(obj, 'properties')):
            return True
        return False

    @staticmethod
    def createSpeckleMesh():
        attr = {'type':'Mesh','_id':None, 'hash':1234, 'geometryHash':None,'applicationId':'Blender', 'properties':None,
        'vertices':[], 'faces':[], 'colors':[]}
        return SpeckleResource(attr)

    @staticmethod
    def isSpecklePlaceholder(obj):
        print ("TODO: implement SpeckleResource.isSpecklePlaceholder")
        return

    @staticmethod
    def createSpecklePlaceholder(id = None):
        attr = {'type':'Placeholder','_id':id}
        return SpeckleResource(attr)

    @staticmethod
    def isSpeckleLayerProperties(obj):
        print ("TODO: implement SpeckleResource.isSpeckleLayerProperties")
        return

    @staticmethod
    def createSpeckleLayerProperties():
        attr = {'color':'','visible':True, 'pointsize':1.0, 'linewidth':0.25, 'shininess':0.5, 
        'smooth':False,'showEdges':True, 'wireframe':True}
        return SpeckleResource(attr)

    @staticmethod
    def isSpeckleLayer(obj):
        print ("TODO: implement SpeckleResource.isSpeckleLayer")
        return

    @staticmethod
    def createSpeckleLayer():
        attr = {'name':'SpeckleLayer', 'guid':None, 'orderIndex':0, 'startIndex':0, 'objectCount':0, 
        'topology':'', 'properties':SpeckleResource.createSpeckleLayerProperties()}
        return SpeckleResource(attr)

    @staticmethod
    def isSpeckleStream(obj):
        print ("TODO: implement SpeckleResource.isSpeckleStream")
        return

    @staticmethod
    def createSpeckleStream():
        attr = {'streamId':'', 'name':'SpeckleStream', 'objects':[], 'layers':[], 'baseProperties':None, 
        'globalMeasures':None, 'isComputedResult':False, 'viewerLayers':[], 'parent':'', 
        'children':[], 'ancestors':[]}
        return SpeckleResource(attr)

    @staticmethod
    def createSpeckleStreamUpdate():
        attr = {'objects':[], 'layers':[]}
        return SpeckleResource(attr)


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
	name="speckle", 
	description="Python client for Speckle framework.",
    long_description=long_description,
    long_description_content_type="text/markdown",	
	url="https://github.com/speckleworks/SpeckleBlender",
	author="Tom Svilans",
	author_email="tom.svilans@gmail.com",
	license="MIT",
	version="0.1.4",
	packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],	
	)